/*
 * restrict_thread_drv driver -
 *
 *  Copyright 2010 Sony Corporation.
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation, version 2.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __RESTRICT_THREAD_H__
#define __RESTRICT_THREAD_H__

#include "rest_thread_if.h"
#include <linux/list.h>
#include <linux/cdev.h>


typedef struct ST_ACTIVE_THREAD_INFO {
    pid_t pid;
    cputime_t total_active_time;
    unsigned long status;
} active_thread_info_t;

typedef struct ST_ACTIVE_THREAD_ENTRY {
    struct list_head list;
    active_thread_info_t active_thread_info;
} active_thread_entry_t;

typedef struct ST_PID_ENTRY {
    pid_t pid;
    struct list_head list;
} pid_entry_t;

typedef struct ST_SCHED_THD_IONFO {
    pid_t pid;
    wait_queue_head_t wait_queue;
    unsigned long state_flag;
    unsigned long flag;  /* use wait_event*/
} sched_thd_info_t;

typedef struct ST_RESTRICT_THREAD_INFO {
    struct cdev cdev;
    rest_thd_set_time_t rest_time;
    unsigned long search_thread_group_jif;

    struct list_head active_list;
    spinlock_t       active_list_lock;
    atomic_t         active_cnt;

    struct list_head restrict_wait_list;
    spinlock_t       restrict_wait_list_lock;
    atomic_t         restrict_wait_cnt;

    struct list_head restrict_done_list;
    spinlock_t       restrict_done_list_lock;
    atomic_t         restrict_done_cnt;

    struct kmem_cache *active_cache;
    struct kmem_cache *pid_entry_cache;

    unsigned long    rest_thread_flag;

    unsigned long next_restrict_reset_jif;
    unsigned long next_search_thread_group_jif;


    sched_thd_info_t sched_thread_info;

    pid_t         parent_pid;
    int set_policy;
    struct sched_param set_sched_param;

    struct timer_list rest_timer_list;

    //debug
    unsigned int debug_total_reset_cnt;
    unsigned int debug_total_restrict_cnt;

    unsigned int debug_set_priority;
    rest_thd_set_time_t debug_rest_time;
    unsigned long debug_search_jif;
} restrict_thread_info_t;


//st_rest_thd_thread_info_t.status
#define DEF_REST_THD_STATUS_ALREADY_RESTRICT 0x0

//st_watch_thd_info_t.state_flag.
#define DEF_REST_THD_STATE_FLAG_RESTRICT             0x0
#define DEF_REST_THD_STATE_FLAG_RESET_THREAD         0x1
#define DEF_REST_THD_STATE_FLAG_SEARCH_THREAD_GROUP  0x2


#define DEF_REST_THD_START_TIMER 0x0
#define DEF_REST_THD_STOP_REST_THREAD 0x1
#define DEF_REST_THD_FORCE_STOP_MODE 0x2
#define DEF_REST_THD_NO_RESTRICT_MODE 0x3
#define DEF_REST_THD_SET_DEBUG_PARAM 0x4


#define DEF_REST_THD_SCHED_THD_WAKEUP 0x0

#endif /*__RESTRICT_THREAD_H__*/

